-----------------------------------------------------------------------------------------------
-- Client Lua Script for Pets
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- Pets Module Definition
-----------------------------------------------------------------------------------------------
local Pets = {
	allpets = nil,
	knowpets = nil
} 
local kAPIVersion = Apollo.GetAPIVersion()

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Pets:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	self.allpets = {}
	self.knowpets = {}
	self.lastrandom = nil
	self.size = 6
	self.icon = true

    return o
end

function Pets:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- Pets OnLoad
-----------------------------------------------------------------------------------------------
function Pets:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("Pets.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)	
	
end

-----------------------------------------------------------------------------------------------
-- Pets OnDocLoaded
-----------------------------------------------------------------------------------------------
function Pets:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "Pets", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end			
		
		if self.Config ~= nil then			
			if self.Config.PositionMain ~= nil then
				self.wndMain:SetAnchorOffsets(
			      	self.Config.PositionMain.Left,
			      	self.Config.PositionMain.Top,
			      	self.Config.PositionMain.Right,
			      	self.Config.PositionMain.Bottom)
				self.wndMain:Show(true, true)
			end			
			if self.Config.PositionMain == nil then
				self.wndMain:Show(true, true)
			end			
			if self.Config.Columns == nil then
				self.Config.Columns = self.size
			end

		else 
			self.wndMain:Show(true, true)
			self.Config = {}
			self.Config.PositionMain = nil
			self.Config.Columns = self.size
			self.Config.icon = true
		end
		
		self.wndMain:FindChild("List"):Show(false, true)
		
		Apollo.RegisterSlashCommand("pets", "OnPetsOn", self)
		self:LoadPets(0)
	end
end

-- Save

function Pets:OnSave(eLevel)
	local result = nil
 	if eLevel == GameLib.CodeEnumAddonSaveLevel.Character then
  		result = self.Config		
 	end
 	return result
end

function Pets:OnRestore(eLevel, savedData)
	if eLevel == GameLib.CodeEnumAddonSaveLevel.Character then
		if savedData ~= nil then
     		self.Config = savedData
 	 	end
 	end
end

function Pets:WindowMove( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom )
	self.Config.PositionMain = self:GetPosition(self.wndMain)
end

function Pets:GetPosition(form)
 	local result = nil
 	if form ~= nil then
  		local leftAnchor, topAnchor, rightAnchor, bottomAnchor = form:GetAnchorOffsets()
  		result = {
				Bottom = bottomAnchor,
				Left = leftAnchor,
				Right = rightAnchor,
				Top = topAnchor,
				}
 	end
 	return result
end

-----------------------------------------------------------------------------------------------
-- Pets Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/pets"
function Pets:OnPetsOn(strCmd, strParam)

	local tAllParams = {}
	
	for sOneParam in string.gmatch(strParam, "[^%s]+") do
		table.insert(tAllParams, sOneParam)
	end
	
	if tAllParams[1] == nil then
		Print("-----------------------------------")
		Print("Pets Options")
		Print("-----------------------------------")
		Print("/pets show - List of Pets with IDs")
		Print("/pets sum <id> - Summon Pet with ID")
		Print("-----------------------------------")
		Print("/pets mode - Switch Mode Text/Icon")
		Print("/pets columns <value> - #(1-12)")
		Print("-----------------------------------")		
	end
	
	
	if tAllParams[1] == "show" then
		self:LoadPets(1)
	end
	
	if tAllParams[1] == "sum" then
	
		self:LoadPets(0)		
		if self.knowpets[tonumber(tAllParams[2])] ~= nil then
			--Print("Summoning "..self.knowpets[tonumber(tAllParams[2])])
			GameLib.SummonVanityPet(tAllParams[2])
		end
		return						
	end
	
	if tAllParams[1] == "rand" then
		self:RandomPet(self:rand(1,#self.allpets))
		return
	end
	
	if tAllParams[1] == "columns" then
		local number = tonumber(tAllParams[2])
		if number < 13 and number > 0 then
			self.Config.Columns = tonumber(tAllParams[2])
		else
			Print("Error")
			Print("/pets columns <value> - #(1-12)")
		end
	end
	
	if tAllParams[1] == "mode" then
		if self.Config.icon then
			self.Config.icon = false
			Print("Pets: TextMode")	
		else
			self.Config.icon = true
			Print("Pets: IconMode")	
		end 		
	end
	
end

function Pets:Debug()

end

function Pets:LoadPets(option)
	self.allpets = self:GetVanityPets()
	
	local size = table.getn(self.allpets)
	table.sort(self.allpets, function(a,b) return (a.bIsKnown and not b.bIsKnown) or (a.bIsKnown == b.bIsKnown and a.strName < b.strName) end)
	
	self.knowpets = {}	
	for i = 1, size, 1 do
		if	self.allpets[i].bIsKnown then
			if option == 1 then
				Print(self.allpets[i].nId .." \t\t".. self.allpets[i].strName)
			end		
			table.insert(self.knowpets , tostring(self.allpets[i].nId), tostring(self.allpets[i].strName))
		end				
	end
end

function Pets:RandomPet(id)
	tPetList = self:GetVanityPets()	
	
	if tPetList[id].bIsKnown and self.lastrand ~= id then
		GameLib.SummonVanityPet(tPetList[id].nId)
		self.lastrand = id
	else
		self:RandomPet(self:rand(1,#self.allpets))
	end	
end

function Pets:rand(nMin,nMax)
	return math.random(nMin,nMax)
end

function Pets:GetVanityPets()
	if kAPIVersion >= 11 then
		return CollectiblesLib.GetVanityPetList()
	else	
		return GameLib.GetVanityPetList()
	end	 
end


-- UI Function
function Pets:FillPets(wnd)
	local wndBtn = wnd:FindChild("Pets")
	local wndList = wnd:FindChild("List")
	local tPetList = self:GetVanityPets()
		
	local nSize = wndList:GetWidth()	
	
	wndList:DestroyChildren()	
	
	table.sort(tPetList, function(a,b) return (a.bIsKnown and not b.bIsKnown) or (a.bIsKnown == b.bIsKnown and a.strName < b.strName) end)
		
	local nCount = 0
	for idx, tPet in pairs(tPetList) do
		if tPet.bIsKnown then
			nCount = nCount + 1
		
			local wndCurr = Apollo.LoadForm(self.xmlDoc, "SummBtn", wndList , self)
			wndCurr:SetText(tPet.strName)
			wndCurr:FindChild("nPetID"):SetText(tPet.nId)
				
		end
	end
	
	local nLeft, nTop, nRight, nBottom = wndList :GetAnchorOffsets()
	wndList:SetAnchorOffsets(nLeft, -(nCount * 24), nRight, nBottom)
	
	wndList:ArrangeChildrenVert()
	
end

function Pets:FillIconPets(wnd)
	local wndBtn = wnd:FindChild("Pets")
	local wndList = wnd:FindChild("List")
	local tPetList = self:GetVanityPets()
	
	local nSize = wndList:GetWidth()	
	
	wndList:DestroyChildren()	
		
	table.sort(tPetList, function(a,b) return (a.bIsKnown and not b.bIsKnown) or (a.bIsKnown == b.bIsKnown and a.strName < b.strName) end)
		
	local nRowCount = 0
	local nPetCount = 1
	
	local nRowMax = round((#tPetList / self.Config.Columns))	
	
	for jdx = 0, nRowMax , 1 do
			
		local wndCurrRow = Apollo.LoadForm(self.xmlDoc, "IconRow", wndList , self)
		wndCurrRow:DestroyChildren()
		local nCount = 0
		
		for idx = nPetCount, (nPetCount+self.Config.Columns-1) , 1 do
			if tPetList[idx].bIsKnown then
				
				nPetCount = nPetCount + 1
							
				local wndCurr = Apollo.LoadForm(self.xmlDoc, "Icon", wndCurrRow , self)				
				wndCurr:FindChild("Sprite"):SetSprite(tPetList[idx].splObject and tPetList[idx].splObject:GetIcon() or "Icon_ItemArmorWaist_Unidentified_Buckle_0001")
				wndCurr:FindChild("nPetID"):SetText(tPetList[idx].nId)								
				nCount = nCount + 1
			end								
		end
		
		wndCurrRow:ArrangeChildrenHorz()		
		nRowCount = nRowCount + 1		
		if not tPetList[nPetCount].bIsKnown then break end		
	end
	
	local nLeft, nTop, nRight, nBottom = wndList :GetAnchorOffsets()
	wndList:SetAnchorOffsets(nLeft, -(nRowCount * 34), nRight, nBottom )
	wndList:ArrangeChildrenVert()	
end

function round(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end


function Pets:SumPet(wndHandler)	
	GameLib.SummonVanityPet(tonumber(wndHandler:FindChild("nPetID"):GetText()))
	self.wndMain:FindChild("List"):Show(false,true)	
end

function Pets:PetListShow(wndHandler, wndControl, eMouseButton, nPosX, nPosY, bDoubleClick)
	
	if eMouseButton == GameLib.CodeEnumInputMouse.Left then
		if not self.wndMain:FindChild("List"):IsShown() then			
			if self.Config.icon then
				self:FillIconPets(self.wndMain)
			else
				self:FillPets(self.wndMain)				
			end
			self.wndMain:FindChild("List"):Show(true, true)
		else
			self.wndMain:FindChild("List"):Show(false, true)
		end
	end

end


-----------------------------------------------------------------------------------------------
-- Pets Instance
-----------------------------------------------------------------------------------------------
local PetsInst = Pets:new()
PetsInst:Init()
